﻿using AutoMapper;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;
using NPOI.SS.Util;
using NPOI.XSSF.UserModel;
using Performance.DtoModels;
using Performance.EntityModels;
using Performance.Infrastructure;
using Performance.Repository;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace Performance.Services
{
    public class PerExcelService : IAutoInjection
    {
        private PerSheetService _perSheetService;
        private PerHeaderService _perHeaderService;
        private PerforPerSheetRepository _perforImSheetRepository;
        private PerforImDataRepository _perforImDataRepository;
        private PerforImHeaderRepository _perforImHeaderRepository;
        private PerforImEmployeeRepository _perforImEmployeeRepository;
        private PerforImaccountbasicRepository _perforImaccountbasicRepository;
        private PerforResAccountnurseRepository _perforResAccountnurseRepository;
        private PerforCofpositionRepository _perforCofpositionRepository;
        private PerforCofdirectorRepository _perforCofdirectorRepository;
        private PerforCofdrugpropRepository _perforCofdrugpropRepository;
        private PerforCofincomeRepository _perforCofincomeRepository;
        private PerforCofworkyearRepository _perforCofworkyearRepository;
        private PerforResAccountdoctorRepository _perforResAccountdoctorRepository;
        private PerforResbaiscnormRepository _perforResbaiscnormRepository;
        private PerforRescomputeRepository _perforRescomputeRepository;
        private PerforImspecialunitRepository _perforImspecialunitRepository;
        private PerforResspecialunitRepository _perforResspecialunitRepository;
        private PerforPerAllotRepository _perforPerAllotRepository;
        public PerExcelService(PerSheetService perSheetService,
            PerHeaderService perHeaderService,
            PerforPerSheetRepository perforImSheetRepository,
            PerforImDataRepository perforImDataRepository,
            PerforImHeaderRepository perforImHeaderRepository,
            PerforImEmployeeRepository perforImEmployeeRepository,
            PerforImaccountbasicRepository perforImaccountbasicRepository,
            PerforResAccountnurseRepository perforResAccountnurseRepository,
            PerforCofpositionRepository perforCofpositionRepository,
            PerforCofdirectorRepository perforCofdirectorRepository,
            PerforCofdrugpropRepository perforCofdrugpropRepository,
            PerforCofincomeRepository perforCofincomeRepository,
            PerforResAccountdoctorRepository perforResAccountdoctorRepository,
            PerforResbaiscnormRepository perforResbaiscnormRepository,
            PerforRescomputeRepository perforRescomputeRepository,
            PerforCofworkyearRepository perforCofworkyearRepository,
            PerforImspecialunitRepository perforImspecialunitRepository,
            PerforResspecialunitRepository perforResspecialunitRepository,
            PerforPerAllotRepository perforPerAllotRepository)
        {
            _perSheetService = perSheetService;
            _perHeaderService = perHeaderService;
            _perforImSheetRepository = perforImSheetRepository;
            _perforImDataRepository = perforImDataRepository;
            _perforImHeaderRepository = perforImHeaderRepository;
            _perforImEmployeeRepository = perforImEmployeeRepository;
            _perforImaccountbasicRepository = perforImaccountbasicRepository;
            _perforResAccountnurseRepository = perforResAccountnurseRepository;
            _perforCofpositionRepository = perforCofpositionRepository;
            _perforResAccountdoctorRepository = perforResAccountdoctorRepository;
            _perforCofdirectorRepository = perforCofdirectorRepository;
            _perforCofdrugpropRepository = perforCofdrugpropRepository;
            _perforCofincomeRepository = perforCofincomeRepository;
            _perforCofworkyearRepository = perforCofworkyearRepository;
            _perforResbaiscnormRepository = perforResbaiscnormRepository;
            _perforRescomputeRepository = perforRescomputeRepository;
            _perforPerAllotRepository = perforPerAllotRepository;
            _perforImspecialunitRepository = perforImspecialunitRepository;
            _perforResspecialunitRepository = perforResspecialunitRepository;
        }

        public void Execute(per_allot allot)
        {
            allot.States = 2;
            _perforPerAllotRepository.Update(allot);

            Clear(allot.ID);

            // 导出数据
            var excel = Import(allot.Path);

            // 保存数据
            Save(excel.PerSheet, allot.ID, 1);

            // 计算合并数据
            List<PerSheet> list = ProcessCompute(excel);

            // 保存过程数据
            Save(list, allot.ID, 2);

            var baiscnorm = NurseBaiscnorm(list);

            // 计算最总数据
            Compute(allot, excel, baiscnorm);
            SpecialUnitCompute(excel, allot);

            //发送邮件
            SendEmail(allot);

            allot.States = 5;
            _perforPerAllotRepository.Update(allot);
        }

        /// <summary>
        /// 特殊科室绩效计算
        /// </summary>
        /// <param name="excel"></param>
        /// <param name="allot"></param>
        private void SpecialUnitCompute(PerExcel excel, per_allot allot)
        {
            var specialUnit = excel.PerSheet.FirstOrDefault(t => t.SheetType == SheetType.SpecialUnit);
            if (specialUnit == null || specialUnit.PerData.Count == 0)
                return;
            var dataList = specialUnit.PerData.Select(t => (PerDataSpecialUnit)t);

            var resDataList = dataList.Select(t => new res_specialunit
            {
                AllotID = allot.ID,
                AccountingUnit = t.AccountingUnit,
                Department = t.AccountingUnit,
                Number = t.Number,
                QuantitativeIndicators = t.QuantitativeIndicators,
                Quantity = t.Quantity,
                QuantitativeIndicatorsValue = t.QuantitativeIndicatorsValue,
                ScoringAverage = t.ScoringAverage,
                OtherPerfor = t.OtherPerfor,
                Punishment = t.Punishment,
                Adjust = t.Adjust,
                Avg = t.Quantity * t.QuantitativeIndicatorsValue / t.Number,
                ShouldFee = t.Quantity * t.QuantitativeIndicatorsValue,
                GiveFee = (t.Quantity * t.QuantitativeIndicatorsValue + t.OtherPerfor + t.Punishment) * t.Adjust,
            });
            _perforResspecialunitRepository.AddRange(resDataList.ToArray());
        }

        /// <summary>
        /// 获取临床护士平均绩效
        /// </summary>
        /// <param name="list"></param>
        /// <returns></returns>
        private res_baiscnorm NurseBaiscnorm(List<PerSheet> list)
        {
            var sheet = list.FirstOrDefault(t => t.SheetType == SheetType.ComputeNurseAccount);
            var perdata = sheet.PerData.Select(t => (PerDataAccountNurse)t);
            return new res_baiscnorm
            {
                PositionName = "临床护士",
                TotelNumber = perdata.Sum(t => t.Number),
                TotelValue = perdata.Sum(t => t.PerforTotal),
                AvgValue = perdata.Sum(t => t.PerforTotal) / perdata.Sum(t => t.Number)
            };
        }

        private void SendEmail(per_allot allot)
        {

        }

        public void Clear(int allotId)
        {
            var director = _perforCofdirectorRepository.GetEntities(t => t.AllotID == allotId);
            if (director != null)
                _perforCofdirectorRepository.RemoveRange(director.ToArray());
            var durgprop = _perforCofdrugpropRepository.GetEntities(t => t.AllotID == allotId);
            if (durgprop != null)
                _perforCofdrugpropRepository.RemoveRange(durgprop.ToArray());
            var income = _perforCofincomeRepository.GetEntities(t => t.AllotID == allotId);
            if (income != null)
                _perforCofincomeRepository.RemoveRange(income.ToArray());
            var position = _perforCofpositionRepository.GetEntities(t => t.AllotID == allotId);
            if (position != null)
                _perforCofpositionRepository.RemoveRange(position.ToArray());
            var workyear = _perforCofworkyearRepository.GetEntities(t => t.AllotID == allotId);
            if (workyear != null)
                _perforCofworkyearRepository.RemoveRange(workyear.ToArray());
        }

        /// <summary>
        /// 导入excel数据
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public PerExcel Import(string path)
        {
            PerExcel excel = new PerExcel
            {
                Path = path,
                FileName = FileHelper.GetFileNameNoExtension(path),
                Version = FileHelper.GetExtension(path) == ".xlsx" ? ExcelVersion.xlsx : ExcelVersion.xls,
                PerSheet = new List<PerSheet>()
            };
            using (FileStream fs = new FileStream(path, FileMode.Open))
            {
                IWorkbook workbook = (excel.Version == ExcelVersion.xlsx)
                    ? (IWorkbook)(new XSSFWorkbook(fs))
                    : (IWorkbook)(new HSSFWorkbook(fs));
                for (int i = 0; i < workbook.NumberOfSheets; i++)
                {
                    var sheet = workbook.GetSheetAt(i);
                    if (SheetType.Unidentifiable != _perSheetService.GetSheetType(sheet.SheetName))
                    {
                        var st = _perSheetService.Sheet(sheet);
                        excel.PerSheet.Add(st);
                    }
                }
                return excel;
            }
        }


        /// <summary>
        /// 复制报表基础配置
        /// </summary>
        /// <param name="iD"></param>
        public void Copy(per_allot allot)
        {
            var list = _perforPerAllotRepository.GetEntities(t => t.HospitalId == allot.HospitalId)
                        .OrderByDescending(t => t.Year).ThenBy(t => t.Month).ToList();
            int allotId = 0;
            for (int i = 0; i < list.Count; i++)
            {
                if (list[i].ID == allot.ID && (i - 1) >= 0)
                    allotId = list[i - 1].ID;
            }
            if (allotId > 0)
            {
                var director = _perforCofdirectorRepository.GetEntities(t => t.AllotID == allotId);
                if (director != null)
                {
                    director.ForEach(t => { t.ID = 0; t.AllotID = allot.ID; });
                    _perforCofdirectorRepository.AddRange(director.ToArray());
                }
                var durgprop = _perforCofdrugpropRepository.GetEntities(t => t.AllotID == allotId);
                if (durgprop != null)
                {
                    durgprop.ForEach(t => { t.ID = 0; t.AllotID = allot.ID; });
                    _perforCofdrugpropRepository.AddRange(durgprop.ToArray());
                }
                var income = _perforCofincomeRepository.GetEntities(t => t.AllotID == allotId);
                if (income != null)
                {
                    income.ForEach(t => { t.ID = 0; t.AllotID = allot.ID; });
                    _perforCofincomeRepository.AddRange(income.ToArray());
                }
                var position = _perforCofpositionRepository.GetEntities(t => t.AllotID == allotId);
                if (position != null)
                {
                    position.ForEach(t => { t.ID = 0; t.AllotID = allot.ID; });
                    _perforCofpositionRepository.AddRange(position.ToArray());
                }
                var workyear = _perforCofworkyearRepository.GetEntities(t => t.AllotID == allotId);
                if (workyear != null)
                {
                    workyear.ForEach(t => { t.ID = 0; t.AllotID = allot.ID; });
                    _perforCofworkyearRepository.AddRange(workyear.ToArray());
                }
            }
        }

        /// <summary>
        /// 计算绩效
        /// </summary>
        /// <param name="excel"></param>
        public List<PerSheet> ProcessCompute(PerExcel excel)
        {
            return _perSheetService.ProcessCompute(excel);
        }


        /// <summary>
        /// 保存绩效结果
        /// </summary>
        /// <param name="perSheets"></param>
        /// <param name="allotId"></param>
        /// <param name="source">数据来源 1 excel 导入 2 计算</param>
        public void Save(List<PerSheet> perSheets, int allotId, int source)
        {
            foreach (var sheet in perSheets)
            {
                var imsheet = new per_sheet { AllotID = allotId, SheetName = sheet.SheetName, Source = source, SheetType = (int)sheet.SheetType };
                _perforImSheetRepository.Add(imsheet);
                if (sheet.SheetType == SheetType.Employee)
                {
                    var dataList = sheet.PerData.Select(t => (PerDataEmployee)t);
                    foreach (var data in dataList)
                    {
                        var imdata = Mapper.Map<im_employee>(data);
                        imdata.SheetID = imsheet.ID;
                        imdata.AllotID = allotId;
                        _perforImEmployeeRepository.Add(imdata);
                    }
                }
                else if (sheet.SheetType == SheetType.DeptAccounting)
                {
                    var dataList = sheet.PerData.Select(t => (PerDataAccountBaisc)t);
                    List<im_accountbasic> addList = new List<im_accountbasic>();
                    foreach (var data in dataList)
                    {
                        var imdata = Mapper.Map<im_accountbasic>(data);
                        imdata.SheetID = imsheet.ID;
                        imdata.AllotID = allotId;
                        addList.Add(imdata);
                    }
                    _perforImaccountbasicRepository.AddRange(addList.ToArray());
                }
                else if (sheet.SheetType == SheetType.SpecialUnit)
                {
                    var dataList = sheet.PerData.Select(t => (PerDataSpecialUnit)t);
                    List<im_specialunit> addList = new List<im_specialunit>();
                    foreach (var data in dataList)
                    {
                        var imdata = Mapper.Map<im_specialunit>(data);
                        imdata.SheetID = imsheet.ID;
                        imdata.AllotID = allotId;
                        addList.Add(imdata);
                    }
                    _perforImspecialunitRepository.AddRange(addList.ToArray());
                }
                else if (sheet.SheetType == SheetType.ComputeDoctorAccount)
                {
                    var dataList = sheet.PerData.Select(t => (PerDataAccountDoctor)t);
                    List<res_accountdoctor> addList = new List<res_accountdoctor>();
                    foreach (var data in dataList)
                    {
                        var imdata = Mapper.Map<res_accountdoctor>(data);
                        imdata.SheetID = imsheet.ID;
                        imdata.AllotID = allotId;
                        addList.Add(imdata);
                    }
                    _perforResAccountdoctorRepository.AddRange(addList.ToArray());
                }
                else if (sheet.SheetType == SheetType.ComputeNurseAccount)
                {
                    var dataList = sheet.PerData.Select(t => (PerDataAccountNurse)t);
                    List<res_accountnurse> addList = new List<res_accountnurse>();
                    foreach (var data in dataList)
                    {
                        var imdata = Mapper.Map<res_accountnurse>(data);
                        imdata.SheetID = imsheet.ID;
                        imdata.AllotID = allotId;
                        addList.Add(imdata);
                    }
                    _perforResAccountnurseRepository.AddRange(addList.ToArray());
                }
                else
                {
                    List<im_header> addHeadList = new List<im_header>();
                    foreach (var header in sheet.PerHeader)
                    {
                        var imheader = Mapper.Map<im_header>(header);
                        imheader.SheetID = imsheet.ID;
                        imheader.AllotID = allotId;
                        addHeadList.Add(imheader);
                        if (header.IsHasChildren)
                        {
                            foreach (var child in header.Children)
                            {
                                var imheaderChild = Mapper.Map<im_header>(child);
                                imheaderChild.SheetID = imsheet.ID;
                                imheaderChild.ParentID = imheader.ID;
                                addHeadList.Add(imheaderChild);
                            }
                        }
                    }
                    _perforImHeaderRepository.AddRange(addHeadList.ToArray());

                    List<im_data> addDataList = new List<im_data>();
                    var dataList = sheet.PerData.Select(t => (PerData)t);
                    foreach (var data in dataList)
                    {
                        var imdata = Mapper.Map<im_data>(data);
                        imdata.SheetID = imsheet.ID;
                        imdata.AllotID = allotId;
                        addDataList.Add(imdata);
                    }
                    _perforImDataRepository.AddRange(addDataList.ToArray());
                }
            }

            //throw new NotImplementedException();
        }

        public IWorkbook ExportCompute(List<PerSheet> sheetList)
        {
            IWorkbook workbook = new XSSFWorkbook();
            var cellstyle = workbook.CreateCellStyle();
            cellstyle.VerticalAlignment = VerticalAlignment.Center;
            cellstyle.Alignment = HorizontalAlignment.Center;

            foreach (var sheet in sheetList)
            {
                ISheet exportSheet = workbook.CreateSheet(sheet.SheetName);
                //创建列头行
                IRow row = null, childRow = null;
                foreach (var header in sheet.PerHeader.OrderBy(t => t.PointCell))
                {
                    row = CreateRow(cellstyle, exportSheet, row, header);
                    //创建二级列头
                    if (header.IsHasChildren)
                    {
                        foreach (var child in header.Children.OrderBy(t => t.PointCell))
                        {
                            childRow = CreateRow(cellstyle, exportSheet, childRow, child);
                        }
                    }
                }
                //反转列头
                var headList = _perHeaderService.GetPerHeaderReverse(sheet.PerHeader);
                var dataList = sheet.PerData.Select(t => (PerData)t);
                var maxrow = headList.Max(t => t.PointRow);
                //循环核算单元 创建数据行
                var accountingUnitList = dataList.OrderBy(t => t.RowNumber).Select(t => t.AccountingUnit).Distinct();
                for (int i = 0; i < accountingUnitList.Count(); i++)
                {
                    var accountingUnit = accountingUnitList.ElementAt(i);
                    IRow dataRow = exportSheet.CreateRow(i + maxrow + 2);
                    dataRow.CreateCell(0).SetCellValue(accountingUnit);
                    dataRow.GetCell(0).CellStyle = cellstyle;
                    foreach (var header in headList.OrderBy(t => t.PointCell))
                    {
                        var cellValue = dataList.FirstOrDefault(t => t.AccountingUnit == accountingUnit && t.TypeName == header.CellValue)?.CellValue;
                        if (cellValue.HasValue && cellValue.Value > 0)
                        {
                            dataRow.CreateCell(header.PointCell).SetCellValue(Convert.ToDouble(cellValue.Value));
                            dataRow.GetCell(header.PointCell).CellStyle = cellstyle;
                        }
                    }
                }
            }
            return workbook;
        }

        /// <summary>
        /// 创建行
        /// </summary>
        /// <param name="cellstyle"></param>
        /// <param name="exportSheet"></param>
        /// <param name="row"></param>
        /// <param name="header"></param>
        /// <returns></returns>
        private IRow CreateRow(ICellStyle cellstyle, ISheet exportSheet, IRow row, PerHeader header)
        {
            if (header.IsMerge)
            {
                var cellRange = new CellRangeAddress(header.PointRow, header.PointRow + header.MergeRow - 1, header.PointCell, header.PointCell + header.MergeCell - 1);
                exportSheet.AddMergedRegion(cellRange);
            }
            row = row ?? exportSheet.CreateRow(header.PointRow);
            row.CreateCell(header.PointCell).SetCellValue(header.CellValue);
            row.GetCell(header.PointCell).CellStyle = cellstyle;
            return row;
        }

        /// <summary>
        /// 计算最终数据
        /// </summary>
        /// <param name="excel"></param>
        public void Compute(per_allot allot, PerExcel excel, res_baiscnorm baiscnorm)
        {
            var sheetList = _perforImSheetRepository.GetEntities(t => t.AllotID == allot.ID);
            //取出人员信息           
            var empolyeeList = _perforImEmployeeRepository.GetEntities(t => t.AllotID == allot.ID);
            //人员身份字典
            var positionList = _perforCofpositionRepository.GetEntities();
            //年资系数
            var workyearList = _perforCofworkyearRepository.GetEntities();
            //规模绩效和效率绩效配置表
            var directorList = _perforCofdirectorRepository.GetEntities();
            //取出医生科室
            var doctorList = _perforResAccountdoctorRepository.GetEntities(t => t.AllotID == allot.ID);
            //取出护士科室
            var nurseList = _perforResAccountnurseRepository.GetEntities(t => t.AllotID == allot.ID);

            List<ComputeEmployee> computeEmployees = Mapper.Map<List<ComputeEmployee>>(empolyeeList);

            List<ComputeSource> computeSources = new List<ComputeSource>();
            computeSources.AddRange(Mapper.Map<List<ComputeSource>>(doctorList));
            computeSources.AddRange(Mapper.Map<List<ComputeSource>>(nurseList));

            ComputeDirector computeDirector = new ComputeDirector();
            var computResult = computeDirector.Compute(computeEmployees, computeSources, directorList);
            var baiscnormList = computeDirector.ComputeAvg(computResult);
            baiscnormList.Add(baiscnorm);

            var computResult2 = computeDirector.Compute(computeEmployees, baiscnormList, workyearList);


            var computes = Mapper.Map<List<res_compute>>(computResult);
            computes.AddRange(Mapper.Map<List<res_compute>>(computResult2));
            computes.ForEach(t => t.AllotID = allot.ID);
            _perforRescomputeRepository.AddRange(computes.ToArray());

            baiscnormList.ForEach(t => t.AllotID = allot.ID);
            _perforResbaiscnormRepository.AddRange(baiscnormList.ToArray());
        }
    }
}
